//==================================================================//
//		Game.h														//									
//		Created by O. Reshta (http://www.brigsoft.com), July 2001.	// 
//																	//
//		You can use and modify this file free on terms:				//	
//		1) Don't remove this title.									//
//		2) You will take all responsibility							//
//		   for project functionality after changing this file.		//
//																	//
//		Recommendation: You would insert "Modified  by " comment	//
//						if you will modify this file.				//
//==================================================================//

//Modified by:


#ifndef CSHAPE_H
#define CSHAPE_H

//---------------------------------------------------------------------//
//  These types, functions and constants are shared by other classes.  //
//---------------------------------------------------------------------//

// Moving and turning directions.
enum TDir {Non, Lf, Up, Rt, Dn };

// Helpful drawing function.
void DrawColorRect(HDC hDC, RECT* pRect, COLORREF color);

const int gnRows	= 27;	// Number of game field rows.
const int gnCols	= 40;	// Number of game field columns.
const int gnCellSize = 15;	// Size of cell in pixels.


//----------------------------------------------//
//  These classes execute main game functions.	//
//----------------------------------------------//

// Messagegs for try/throw/catch are sent when snake is dieing. 
class StopWall {};
class StopHedgehog {};
class StopBody {};


// - CCell -------------------------------------------
// A cell of game field.
// It stores snake units, mice, hedgehogs, moving directions
// and draws items.

class CCell
{

public:
	
	enum TCellType {Space, Head, Tail, Body, BlueMouse, VioletMouse, RedMouse, Hedgehog };

	CCell();

	TCellType	GetType() const;
	TDir		GetDir() const;

	void  SetType(const TCellType Type);
	void  SetDir(const TDir	Dir);

	void Invalidate(int Row, int Col) const;
	void Draw(HDC hDC, int Row, int Col) const;

	void TurnLeft();
	void TurnRight();

	static HBITMAP m_hSnakeBmp; // Shared by all cells.
								// Is downloaded by CPlayBox::Init().
								// Is deleted by CPlayBox::~CPlayBox()

private:

	TCellType	m_Type;
	TDir		m_Dir;

};

//----------------------------------------------------
	
inline void  CCell::SetType(const TCellType Type)
{
	m_Type = Type;
}

inline void  CCell::SetDir(const TDir Dir)
{
	m_Dir = Dir;
}

inline CCell::TCellType CCell::GetType() const
{
	return m_Type; 
}


inline TDir CCell::GetDir() const
{
	return m_Dir; 
}

// - CPnt -----------------------------------------------
// Some point of field. It is used as reference to cell.
class  CPnt
{

public:

	int row;
	int col;

	void MoveToDir(TDir Dir);
};

//- CSnake ---------------------------------------------
// Main snake functions. Snake is not stored at all.
// All information is stored in cells.
// Snake includes info about tail and heard coordinates,
// moves tail and head changing correspondent cells,
// tests cells looking for mice, hedgehogs and its body.

class CSnake
{

public:
	CSnake();
	void Init();

	void Grow();
	void Move();
	void TestFront();

	void TurnLeft();
	void TurnRight();

	int  GetGrowLen() const;
	void DecGrowLen();

	int  GetLen() const;
	void IncLen();

private:
	
	CPnt		m_Head;
	CPnt		m_Tail;
	const int	m_StartL;
	int			m_nGrowLen;
	int			m_nLen;
};

// ----------------------------------------------

inline int CSnake::GetGrowLen() const
{
	return m_nGrowLen;
}

inline void CSnake::DecGrowLen()
{
	-- m_nGrowLen;
	ASSERT(m_nGrowLen >=0);
}

inline int CSnake::GetLen() const
{
	return m_nLen;
}

inline void CSnake::IncLen()
{
	m_nLen ++;
}


//- CMouse -----------------------------------------------------
// Class generates mice and hedgehogs(!) and stores them in cells.

class CMouse
{
public : 

	CMouse();
	void Generate();

private:
	int m_nTicker;

};

// ------------------------------------------------------

inline CMouse::CMouse()
{
	srand(::GetTickCount());
	m_nTicker = -1;
}

//- CPlayBox ----------------------------------------------
// Game field. It consists of cells and calls cells drawing.
// It executes game steps and calls snake and mouse functions.
// Play box object is global and is called from engine.

class CPlayBox//: public CShape
{
public:

	CPlayBox();
	~CPlayBox();

	void TurnSnakeLeft();
	void TurnSnakeRight();

	void Draw(HDC hDC);

	void NextTimeStep();

	void Init();

	int GetSnakeLen() const;

private:

	CSnake m_Snake;
	CMouse m_Mouse;		// It is not a mouse. It is mice generator.

};

//----------------------------------------------------

inline int CPlayBox::GetSnakeLen() const
{
	return m_Snake.GetLen();
}

inline void CPlayBox::TurnSnakeLeft()
{
	m_Snake.TurnLeft();
}

inline void CPlayBox::TurnSnakeRight()
{
	m_Snake.TurnRight();
}

#endif